using System.Windows;
using System.Windows.Media.Animation;

using MindFusion.Diagramming;

namespace AnimateLinks
{
	public class LinkAnimator : DependencyObject
	{
		public LinkAnimator(DiagramLink link, int pointIndex)
		{
			this.link = link;
			this.pointIndex = pointIndex;
		}

		public static DependencyProperty PointProperty = DependencyProperty.Register(
			"Point",
			typeof(Point),
			typeof(LinkAnimator),
			new PropertyMetadata(OnPointChanged));

		private static void OnPointChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var linkAnimator = (LinkAnimator)d;
			var link = linkAnimator.link;
			link.ControlPoints[linkAnimator.pointIndex] = (Point)e.NewValue;
			link.UpdateFromPoints();
		}

		public Point Point
		{
			get { return (Point)GetValue(PointProperty); }
			set { SetValue(PointProperty, value); }
		}

		public void Animate(Point from, Point to)
		{
			var animation = new PointAnimation();
			animation.From = from;
			animation.To = to;

			var storyboard = new Storyboard();
			storyboard.Children.Add(animation);
			Storyboard.SetTarget(animation, this);
			Storyboard.SetTargetProperty(
				animation, new PropertyPath(PointProperty));

			storyboard.Begin();
		}

		private DiagramLink link;
		private int pointIndex;
	}
}